require 'virtus'
require 'elasticsearch'
require 'active_support'
require 'active_support/core_ext'
require 'active_model'
require 'hooks'
require 'leaflet'
require 'hashie/mash'

require 'repobahn/repository'
require 'repobahn/entity'

require 'gummi/version'
require 'gummi/errors'
require 'gummi/configuration'

require 'gummi/db_layer/index'
require 'gummi/db_layer/document/attributes'
require 'gummi/db_layer/document/object'
require 'gummi/db_layer/document'
require 'gummi/db_layer/fields/boolean'
require 'gummi/db_layer/fields/time'
require 'gummi/db_layer/fields/integer'
require 'gummi/db_layer/fields/positive_integer'
require 'gummi/db_layer/fields/keyword'
require 'gummi/db_layer/fields/ngram_and_plain'
require 'gummi/db_layer/fields/path_hierarchy'
require 'gummi/db_layer/fields/string'
require 'gummi/db_layer/fields/sanitized_string'
require 'gummi/db_layer/default_index'
require 'gummi/db_layer/document/search/searching'
require 'gummi/db_layer/document/search/filtered'
require 'gummi/db_layer/document/search/raw'
require 'gummi/db_layer/document/search/result'

require 'gummi/repository_layer/repository'
require 'gummi/repository_layer/repository/result'

require 'gummi/entity_layer/entity'

module Gummi
  def self.env
    if defined? ::Rails
      ::Rails.env
    else
      RAILS_ENV || 'development'
    end
  end

  def self.config
    @config ||= Configuration.new
  end

  def self.configure(&block)
    yield config
  end

  def self.client
    @client ||= ::Elasticsearch::Client.new Gummi.config.attributes
  end

  def self.reload_configuration!
    @client = ::Elasticsearch::Client.new Gummi.config.attributes
  end

end
