module Gummi
  module DbLayer
    module Document
      module Attributes
        extend ActiveSupport::Concern

        module ClassMethods

          def mapping_for_attribute(attribute)
            if attribute.is_a? Virtus::Attribute::EmbeddedValue
              { properties: attribute.primitive.mapping }
            elsif attribute.is_a? Virtus::Attribute::Collection
              mapping_for_attribute(attribute.member_type)
            else
              begin
                attribute.mapping
              rescue NoMethodError => exception
                if exception.message.include?("`mapping'")
                  raise Errors::ImplicitMappingForbidden, "Sorry, you have to explicitly define the attribute type for #{attribute.instance_variable_name rescue nil} in #{self} or maybe you tried to use Array[] where it is not allowed. Original Exception: #{exception.message}"
                else
                  raise exception
                end
              end
            end
          end

          def mapping
            result = {}
            attribute_set.each do |attribute|
              result.merge! attribute.name => mapping_for_attribute(attribute)
            end
            result
          end

        end

      end
    end
  end
end
