module Repobahn
  module Repository
    extend ActiveSupport::Concern

    included do
      include Hooks
      define_hook :after_conversion
    end

    module ClassMethods

      def db_class(*args)
        @db_class = args.first unless args.empty?
        @db_class || default_db_class
      end

      def entity_class(*args)
        @entity_class = args.first unless args.empty?
        @entity_class || default_entity_class
      end

      private

      def default_db_class
        parts = default_entity_class.to_s.split('::')
        parts.insert -2, 'DB'
        parts.join('::').constantize
      end

      def default_entity_class
        name.singularize.constantize
      end

    end

    def db_class
      self.class.db_class
    end

    def entity_class
      self.class.entity_class
    end

    def db_instances_to_entities(db_instances)
      return [] unless db_instances.present?
      entities = Array(db_instances).map { |db_instance| db_instance_to_entity(db_instance) }
      entities.length > 1 ? entities : entities.first
    end

    def db_instance_to_entity(db_instance)
      return nil unless db_instance.present?
      entity = entity_class.new(db_instance.attributes)
      run_hook :after_conversion, entity, db_instance
      entity
    end
  end
end
